<?php
	/*
	* 2018 - VEGA Informatique
	*
	* NOTICE OF LICENSE
	*
	* This source file is subject to the Open Software License (OSL 3.0)
	* that is bundled with this package in the file LICENSE.txt.
	* It is also available through the world-wide-web at this URL:
	* http://opensource.org/licenses/osl-3.0.php
	*
	* DISCLAIMER
	*
	* Do not edit or add to this file if you wish to upgrade PolarisLink to newer
	* versions in the future. 
	*
	*  @author VEGA Informatique <contact@vega-info.fr>
	*  @copyright  2018 VEGA Informatique
	*  @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
	*/

	require_once(_PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkImportBase.php");
	
	/*
	 * ex. : <tag ref="xx">
				<grouptag ref="yy">
					<name />
				</grouptag>
				<name/>
			</tag>
	 */
	class PolarisLinkImport_tag extends PolarisLinkImportBase 
	{	
		protected function OnInit()
		{
			parent::OnInit();
						
			if ($this->Link->MethodTags == 2)
				$this->setCache(new PolarisLinkExtRefCache($this->Link->LinkID, "tag_features"));			
		}
		
		private function getFeatureValueIdByName($parentId, $names)
		{
			foreach ($names as $id_lang => $name)
			{
				$featuresValues = FeatureValue::getFeatureValuesWithLang($id_lang, $parentId);
				foreach ($featuresValues as $value)
					if ($value["value"] == $name)
						return $value["id_feature_value"];
			}
		}
		
		public function getOrCreateTag($polarisRef, $names, $parentId)
		{
			switch ($this->Link->MethodTags)
			{
				case 0:
				case 1:
				case 3:
					// Non géré et par mot clés, on ne passe jamais par ici
					return;
			}			
			
			if (empty($parentId))
				throw new Exception(sprintf("Le parent pour le mot-clé pol-%s n'a pas été trouvé", $polarisRef));
			
			$cache = $this->getCache("pr_tag_features");
			
			$psId = $cache->get($polarisRef);
			if (empty($psId))
			{				
				// On cherche la catégorie par son nom ...
				$names = checkFeatureName($names);

				$psId = $this->getFeatureValueIdByName($parentId, $names);
				
				if (empty($psId))
				{
					$obj = new FeatureValue();
					
					$obj->id_feature = $parentId;
					$obj->value = $this->AutoFillEmptyLangValue($names);
					$obj->save();

					$this->Log($obj, "Création du mot-clé %s", [$obj->value[$this->Link->LangID]]);
					
					$psId = $obj->id;					
				}
				
				$cache->set($polarisRef, $psId);
			}
			
			return $psId;
		}
		
		private function addToProductFeatureVersion($log, $product, $tags)
		{
			$featuresOwned = $this->JobManager->getImporter("grouptag")->GetOwnedTagGroups();

			$currentFeatures = array();
			foreach ($product->getFeatures() as $feature)
				$currentFeatures[$feature["id_feature"]] = $feature["id_feature_value"];
			
			$featuresToHave = array();
			foreach ($tags as $tag)
			{
				$psId = $this->OnImportData($tag);
				if (!empty($psId) && !empty($psId["id"]))
					if (!isset($featuresToHave[$psId["parent"]]) || $featuresToHave[$psId["parent"]] < $psId["id"])
						$featuresToHave[$psId["parent"]] = $psId["id"];
			}
			
			foreach ($featuresOwned as $parent)
			{
				if (!isset($featuresToHave[$parent]))
				{
					if (isset($currentFeatures[$parent]))
					{
						Db::getInstance()->execute('DELETE FROM `'._DB_PREFIX_.'feature_product`
							WHERE `id_product` = '.(int)$product->id.' AND `id_feature_value` = '.(int)$currentFeatures[$parent]
						);

						$feature = new Feature($parent, Configuration::get("PS_LANG_DEFAULT"));
						$featureValue = new Feature($currentFeatures[$parent], Configuration::get("PS_LANG_DEFAULT"));
						$log->LogChange(sprintf("caractéristique %s", $feature->name), null, $featureValue->value);
					}
				}
				else
				{
					$id = $featuresToHave[$parent];
					// Association du mot clé au produit !
					if (!isset($currentFeatures[$parent]) || $currentFeatures[$parent] != $id)
					{
						Product::addFeatureProductImport($product->id, $parent, $id);

						$feature = new Feature($parent, Configuration::get("PS_LANG_DEFAULT"));
						$newFeatureValue = new FeatureValue($id, Configuration::get("PS_LANG_DEFAULT"));
						$oldFeatureValue = isset($currentFeatures[$parent]) ? new FeatureValue($currentFeatures[$parent], Configuration::get("PS_LANG_DEFAULT")) : null;

						$log->LogChange(sprintf("caractéristique %s", $feature->name), $newFeatureValue->value, $oldFeatureValue ? $oldFeatureValue->value : null);
					}
				}
			}
			$log->Commit();
		}
				
		/*
			Les thèmes vont dans les Tags, version facile, il ne s'agit que de texte, par langue
		*/
		private function addToProductTagVersion($log, $product, $data)
		{
			// Chargements des tags de produits
			$currentTags = Tag::getProductTags($product->id);
			if ($currentTags === false)
				$currentTags = [];
			
			$TagsByLang = array();
			foreach ($data as $elt)
			{				
				foreach ($this->getLgValue($elt->name) as $id_lang => $name)
					if ($name != null)
						$tagsByLang[$id_lang][$name] = checkAttributeName($name);
			}
			
			$mustAddTags = count($currentTags) != count($tagsByLang);
			foreach ($tagsByLang as $id_lang => $tags)
			{
				sort($tags);
				$ctags = array();
				if (isset($currentTags[$id_lang]))
					$ctags = $currentTags[$id_lang];
				sort($ctags);
							
				$mustAddTags |= (serialize($tags) != serialize($ctags));
			}
						
			if ($mustAddTags)
			{				
				// Reset ...
				Tag::deleteTagsForProduct($product->id);
				
				// Ok, pour chaque langue, enregistrement sur le produit
				$log->LogChange("mots-clé", join(", ", $tags))->commit();
				foreach ($tagsByLang as $id_lang => $tags)
					Tag::addTags($id_lang, $product->id, $tags);
			}			
		}		
		
		public function addToProduct($log, $product, $data)
		{
			switch ($this->Link->MethodTags)
			{
				case 2; $this->addToProductFeatureVersion($log, $product, $data); break;
				case 3; $this->addToProductTagVersion($log, $product, $data); break;
			}
		}
		
		protected function OnImportData($data)
		{
			if (!$this->Link->OPT_ALLOW_I_CATALOG)
				return;	// Rien à faire ici dans ce cas là !
			
			switch ($this->Link->MethodTags)
			{
				case 0:	// Non géré
				case 3: // Géré par mots clé, ça ne se fait pas ici
					return;
				case 1:	// Géré par catégorie, on reroute
					return $this->JobManager->getImporter("category")->ImportData($data, "grouptag", "tag", "grtag");
			}
			
			// Avons nous la référence Polaris ?
			$polarisRef = (string) $this->GetMandatory($data, "", "ref");

			// Localisation du tag
			$psId = $this->getCache("pr_tag_features")->get($polarisRef);
			
			// Activé ?
			$deleted = (string) $this->GetOptional($data, "", "deleted", "0");
			
			// Parent
			if (empty($data->grouptag))
				throw new Exception("Aucun parent spécifié pour le mot clé alors qu'attendu");
			
			$parentId = $this->JobManager->getImporter("grouptag")->ImportData($data->grouptag);
			if (empty($parentId))
				throw new Exception(sprintf("Impossible d'obtenir le parent pour le mot-clé pol-%s", $polarisRef));
			
			if (!$deleted)
			{
				$obj = new FeatureValue($this->getOrCreateTag($polarisRef, $this->getLgValue($data->name), $parentId));

				$log = $this->NewLogCtx($obj, "Modification du mot-clé");
				$this->updProperty($log, $obj, "value", $data->name, "checkFeatureName");
				$log->commit();

				return array("id" => $obj->id, "parent" => $parentId);
			}
			else if (!empty($psId))
			{		
				// Tag n'existe plus, on supprime
				$obj = new FeatureValue($psId);				
				$obj->delete();
				$this->Log($obj, "Suppression du mot-clé");
			}
		}
	}
	